﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Npgsql;
using System.Collections;

namespace DataGridView
{
    public class Database
    {
        public static NpgsqlConnection conn;
        public static System.Data.DataSet dataSet = null;
        public static NpgsqlDataAdapter dataAdapter = null;
        
        public static bool connect_db()
        {
            Properties.Settings.Default.Reload();
            string connection_string = "Server=" + Properties.Settings.Default.Server + ";"
                + "Port=" + Properties.Settings.Default.Port + ";"
                + "User Id=" + Properties.Settings.Default.User_Id + ";"
                + "Password=" + Properties.Settings.Default.Password + ";"
                + "Database=" + Properties.Settings.Default.Database + ";";
            conn = new NpgsqlConnection(connection_string);
            try
            {
                conn.Open();
                return true;
            }
            catch (Exception ex)
            {
                Console.WriteLine("DataBase Open Error:" + ex.Message);
                return false;
            }
        }

        /// <summary>
        /// 接続確認
        /// </summary>
        /// <returns></returns>
        public static bool CheckConnection()
        {
            bool ret = true;

            while (!connect_db())
            {
                //  接続に失敗した場合は、設定値を入力するウィンドウを表示する
                using (Form2 ff = new Form2())
                {
                    if (ff.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                    {
                        ret = false;
                        break;
                    }
                }
            }

            return ret;
        }

        /// <summary>
        /// 指定されたテーブルのカラム名とコメントを取得します
        /// </summary>
        /// <param name="table_name"></param>
        /// <returns></returns>
        public static Hashtable GetComment(string table_name)
        {
            Hashtable ht = new Hashtable();
            NpgsqlCommand command = new NpgsqlCommand("select "
                + "  attr.attname as column_name, "
                + "  des2.description as column_comment "
                + "from "
                + "  pg_catalog.pg_class a "
                + "    inner join pg_catalog.pg_namespace ns on a.relnamespace = ns.oid "
                + "    inner join pg_catalog.pg_attribute attr on a.oid = attr.attrelid and attr.attnum > 0 "
                + "    left join pg_catalog.pg_description des2 on a.oid = des2.objoid and attr.attnum = des2.objsubid "
                + "where "
                + "  a.relkind IN ('r', 'v') "
                + "  and a.relname = :table_name "
                + "  and ns.nspname IN ('public') "
                , conn);
            command.Parameters.Add(new NpgsqlParameter("table_name", NpgsqlTypes.NpgsqlDbType.Text));
            command.Parameters["table_name"].Value = table_name;

            NpgsqlDataReader dr = command.ExecuteReader();
            while (dr.Read())
            {
                ht.Add(dr["column_name"], dr["column_comment"]);
            }

            return ht;
        }

        public static void GetData()
        {
            dataSet = new System.Data.DataSet();
            dataAdapter = new NpgsqlDataAdapter();
            string sql = "select * from address";

            dataAdapter.SelectCommand = new NpgsqlCommand(sql, Database.conn);

            sql = "insert into address (name, birthday, zipcode, addr, tel) values (:name, :birthday, :zipcode, :addr, :tel)";
            dataAdapter.InsertCommand = new NpgsqlCommand(sql, Database.conn);
            dataAdapter.InsertCommand.Parameters.Add(new NpgsqlParameter("name", NpgsqlTypes.NpgsqlDbType.Text, 0, "name", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.InsertCommand.Parameters.Add(new NpgsqlParameter("birthday", NpgsqlTypes.NpgsqlDbType.Timestamp, 0, "birthday", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.InsertCommand.Parameters.Add(new NpgsqlParameter("zipcode", NpgsqlTypes.NpgsqlDbType.Text, 0, "zipcode", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.InsertCommand.Parameters.Add(new NpgsqlParameter("addr", NpgsqlTypes.NpgsqlDbType.Text, 0, "addr", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.InsertCommand.Parameters.Add(new NpgsqlParameter("tel", NpgsqlTypes.NpgsqlDbType.Text, 0, "tel", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));

            sql = "update address set name = :name, birthday = :birthday, zipcode = :zipcode, addr = :addr, tel = :tel where id = :id";
            dataAdapter.UpdateCommand = new NpgsqlCommand(sql, Database.conn);
            dataAdapter.UpdateCommand.Parameters.Add(new NpgsqlParameter("name", NpgsqlTypes.NpgsqlDbType.Text, 0, "name", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.UpdateCommand.Parameters.Add(new NpgsqlParameter("birthday", NpgsqlTypes.NpgsqlDbType.Timestamp, 0, "birthday", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.UpdateCommand.Parameters.Add(new NpgsqlParameter("zipcode", NpgsqlTypes.NpgsqlDbType.Text, 0, "zipcode", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.UpdateCommand.Parameters.Add(new NpgsqlParameter("addr", NpgsqlTypes.NpgsqlDbType.Text, 0, "addr", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.UpdateCommand.Parameters.Add(new NpgsqlParameter("tel", NpgsqlTypes.NpgsqlDbType.Text, 0, "tel", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Current, DBNull.Value));
            dataAdapter.UpdateCommand.Parameters.Add(new NpgsqlParameter("id", NpgsqlTypes.NpgsqlDbType.Integer, 0, "id", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Original, DBNull.Value));

            sql = "delete from address where id = :id";
            dataAdapter.DeleteCommand = new NpgsqlCommand(sql, Database.conn);
            dataAdapter.DeleteCommand.Parameters.Add(new NpgsqlParameter("id", NpgsqlTypes.NpgsqlDbType.Integer, 0, "id", System.Data.ParameterDirection.Input, false, 0, 0, System.Data.DataRowVersion.Original, DBNull.Value));

            dataAdapter.Fill(dataSet, "address");
        }
    }
}
